@@ -35,7 +35,7 @@ SUPPORTED_RAID = ['0', '1', '10']
 os.environ['LC_NUMERIC'] = "C"
 
 # If you change this please change doc/conf.py as well
-VERSION = '0.3'
+VERSION = '0.4'
 
 # Should the script be run in interactive or non interactive mode ?
 try:
@@ -49,10 +49,10 @@ except KeyError:
 
 
 class Options(object):
-    '''
+    """
     Structure that contains option setting allowing it to be
     passed between parts of ssm.
-    '''
+    """
 
     def __init__(self):
         self.interactive = not SSM_NONINTERACTIVE
@@ -99,10 +99,10 @@ class Struct(object):
 
 
 class StoreAll(argparse._StoreAction):
-    '''
+    """
     Argparse class used to store all valid values. Valid values should not be
     empty or None
-    '''
+    """
 
     def __call__(self, parser, namespace, values, option_string=None):
         for val in values[:]:
@@ -112,10 +112,10 @@ class StoreAll(argparse._StoreAction):
 
 
 class SetBackend(argparse._StoreAction):
-    '''
+    """
     Action for the backend parameter, where we want to store provided
     in SSM_DEFAULT_BACKEND.
-    '''
+    """
 
     def __call__(self, parser, namespace, values, option_string=None):
         # Set default backend to the provided value. All check should be
@@ -126,10 +126,10 @@ class SetBackend(argparse._StoreAction):
 
 
 class FsInfo(object):
-    '''
+    """
     Parse and store information about the file system. Methods specific for
     each file system should be part of this class
-    '''
+    """
 
     def __init__(self, dev, options):
         self.data = {}
@@ -185,9 +185,9 @@ class FsInfo(object):
         self.data['fs_used'] = (bcount - fbcount) * bsize / 1024
 
     def extN_fsck(self):
-        command = ['fsck.{0}'.format(self.fstype), '-f']
+        command = ['fsck.{0}'.format(self.fstype), '-f', '-n']
         if not misc.check_binary(command[0]):
-            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) + \
+            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) +
                     "File system will not be checked")
             return 1
         if self.options.force:
@@ -200,7 +200,7 @@ class FsInfo(object):
     def extN_resize(self, new_size=None):
         command = ['resize2fs', self.device]
         if not misc.check_binary(command[0]):
-            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) + \
+            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) +
                     "File system will not be resized")
             return 1
         if self.options.force:
@@ -238,10 +238,10 @@ class FsInfo(object):
         bsize = int(self.fs_info['blocksize'])
         bcount = int(self.fs_info['dblocks'])
         lbcount = int(self.fs_info['logblocks'])
-        bcount = bcount - lbcount
+        bcount -= lbcount
         agcount = int(self.fs_info['agcount'])
         fbcount = int(self.fs_info['fdblocks'])
-        fbcount = fbcount - (4 + (4 + agcount))
+        fbcount -= 4 + (4 + agcount)
         self.data['fs_size'] = bcount * bsize / 1024
         self.data['fs_free'] = fbcount * bsize / 1024
         self.data['fs_used'] = (bcount - fbcount) * bsize / 1024
@@ -249,7 +249,7 @@ class FsInfo(object):
     def xfs_fsck(self):
         command = ['xfs_check']
         if not misc.check_binary(command[0]):
-            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) + \
+            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) +
                     "File system will not be checked")
             return 1
         if self.options.verbose:
@@ -260,14 +260,14 @@ class FsInfo(object):
     def xfs_resize(self, new_size=None):
         command = ['xfs_growfs', self.device]
         if not misc.check_binary(command[0]):
-            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) + \
+            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) +
                     "File system will not be resized")
             return 1
         if new_size:
             command.insert(1, ['-D', new_size + 'K'])
         if not self.mounted:
             raise Exception("Xfs file system on {0}".format(self.device) +
-                    " has to be mounted to perform an resize.")
+                            " has to be mounted to perform an resize.")
         elif new_size and new_size < self.data['fs_size']:
             raise Exception("Xfs file system can not shrink.")
         else:
@@ -275,7 +275,7 @@ class FsInfo(object):
 
 
 class DeviceInfo(object):
-    '''
+    """
     Parse and store information about the devices present in the system. The
     main source of information are /proc/partitions, /proc/mounts and
     /proc/swaps. self.data should be appended to since there might be other
@@ -285,7 +285,7 @@ class DeviceInfo(object):
     Important thing is that we hide all dm devices here, since they might
     really be a volumes. We let backend decide whether the device should be
     listed as device or not simply by setting 'hide' to True/False.
-    '''
+    """
 
     def __init__(self, options, data=None):
         self.type = 'device'
@@ -342,7 +342,7 @@ class DeviceInfo(object):
                 dev['type'] = 'disk'
 
     def set_globals(self, options):
-        self.options=options
+        self.options = options
 
     def __iter__(self):
         for item in sorted(self.data.iterkeys()):
@@ -356,11 +356,11 @@ class DeviceInfo(object):
 
 
 class Item(object):
-    '''
+    """
     Meta object which provides encapsulation for all devices, pools and
     volumes, so we can work with them as with the usual objects without the
     need to call Dev, Pool or Vol methods directly.
-    '''
+    """
 
     def __init__(self, obj, name):
         self.obj = obj
@@ -425,12 +425,12 @@ class Item(object):
 
 
 class Storage(object):
-    '''
+    """
     Template class to use for storing information about Pools, Volumes and
     Devices from different backends. This simplify things a lot since we do not
     have to manually walk through all the backends, but this class will do this
     for us.
-    '''
+    """
 
     def __init__(self, options):
         self._data = {}
@@ -461,12 +461,12 @@ class Storage(object):
         self.__init__(self.options)
 
     def _apply_prefix_filter(self):
-        '''
+        """
         If SSM_PREFIX FILTER is set, remove all items which basenames does not
         start with SSM_PREFIX_FILTER prefix. This is useful especially for
         testing so that ssm see only relevant devices and does not screw real
         system storage configuration.
-        '''
+        """
         if not SSM_PREFIX_FILTER:
             return
         reg = re.compile("^{0}".format(SSM_PREFIX_FILTER))
@@ -486,7 +486,7 @@ class Storage(object):
         return self._data[name]
 
     def set_globals(self, options):
-        self.options=options
+        self.options = options
         if self._data is None:
             return
         for source in self._data.itervalues():
@@ -498,7 +498,7 @@ class Storage(object):
                 yield item
 
     def ptable(self, cond=None, more_data=None, cond_func=None):
-        '''
+        """
         Print information table about the source (devices, pools, volumes)
         using the predefined variables (below). cond, or cond_func can be
         provided to decide which items not to print out.
@@ -506,7 +506,7 @@ class Storage(object):
         self.header - list of headers for the table
         self.attrs - list of attribute keys to print out
         self.types - types of the attributes to print out (str, or float/int)
-        '''
+        """
         lines = []
         fmt = ""
 
@@ -550,7 +550,7 @@ class Storage(object):
 
         header = [item for item in misc.compress(self.header, columns)]
         alignment = list([(len(self.header[i]))
-                    for i in range(len(self.header))])
+                         for i in range(len(self.header))])
         term_width = misc.terminal_size()[0]
 
         # Update matrix of attribute lengths and construct the final list
@@ -560,7 +560,7 @@ class Storage(object):
             # Find maximum length for each column
             for a, array in enumerate(len_matrix):
                 for i, item in enumerate(array):
-                    if columns[i] == False:
+                    if not columns[i]:
                         alignment[i] = 0
                         continue
                     if item > alignment[i]:
@@ -579,8 +579,8 @@ class Storage(object):
             if length > term_width and \
                (length - term_width) < (alignment[0] - len(header[0])) and \
                line is not None:
-                   alignment[0] = len(header[0])
-                   len_matrix[line][0] = len(header[0])
+                    alignment[0] = len(header[0])
+                    len_matrix[line][0] = len(header[0])
             else:
                 break
 
@@ -621,10 +621,10 @@ class Storage(object):
 
 
 class Pool(Storage):
-    '''
+    """
     Store Pools from all the backends. When new backend is added into the ssm
     it should be registered withing this class with appropriate name.
-    '''
+    """
 
     def __init__(self, *args, **kwargs):
         super(Pool, self).__init__(*args, **kwargs)
@@ -650,7 +650,7 @@ class Pool(Storage):
 
 
 class Devices(Storage):
-    '''
+    """
     Store Devices from all the backends. When new backend is added into the ssm
     it should be registered withing this class with appropriate name.
 
@@ -658,7 +658,7 @@ class Devices(Storage):
     discovered by the DeviceInfo() class then it should just add the
     information into the existing devices by passing the data. But if the
     backed discovers new devices, it should add them as a new entry.
-    '''
+    """
 
     def __init__(self, *args, **kwargs):
         super(Devices, self).__init__(*args, **kwargs)
@@ -683,8 +683,8 @@ class Devices(Storage):
             my_md = Struct()
             my_md.data = {}
 
-        self._data['dev'] = DeviceInfo(data=dict(my_lvm.data.items() + \
-                                                 my_btrfs.data.items() + \
+        self._data['dev'] = DeviceInfo(data=dict(my_lvm.data.items() +
+                                                 my_btrfs.data.items() +
                                                  my_md.data.items()),
                                        options=self.options)
         self.header = ['Device', 'Free', 'Used',
@@ -696,10 +696,10 @@ class Devices(Storage):
 
 
 class Volumes(Storage):
-    '''
+    """
     Store Volumes from all the backends. When new backend is added into the ssm
     it should be registered withing this class with appropriate name.
-    '''
+    """
 
     def __init__(self, *args, **kwargs):
         super(Volumes, self).__init__(*args, **kwargs)
@@ -734,11 +734,11 @@ class Volumes(Storage):
 
 
 class Snapshots(Storage):
-    '''
+    """
     Store Snapshots from all the backends that supports snapshotting. When
     the snapshotting support is added into the backed it should be registered
     within this class with appropriate name.
-    '''
+    """
 
     def __init__(self, *args, **kwargs):
         super(Snapshots, self).__init__(*args, **kwargs)
@@ -754,19 +754,19 @@ class Snapshots(Storage):
             PR.warn(err)
             PR.warn("Can not get information about btrfs snapshots")
 
-        self.header = ['Snapshot', 'Origin', 'Volume size', 'Size',
+        self.header = ['Snapshot', 'Origin', 'Pool', 'Volume size', 'Size',
                        'Type', 'Mount point']
-        self.attrs = ['snap_name', 'origin', 'vol_size', 'snap_size',
-                      'type', 'mount']
-        self.types = [str, str, float, float, str, str]
+        self.attrs = ['dev_name', 'origin', 'pool_name', 'vol_size',
+                      'snap_size', 'type', 'mount']
+        self.types = [str, str, str, float, float, str, str]
         self._apply_prefix_filter()
 
 
 class StorageHandle(object):
-    '''
+    """
     The main class where all the magic is done. All the commands provided by
     ssm have its appropriate functions here which are then called by argparse.
-    '''
+    """
 
     def __init__(self, options=Options()):
         self._mpoint = None
@@ -838,7 +838,7 @@ class StorageHandle(object):
         """
         command = ["mkfs.{0}".format(fstype), volume]
         if not misc.check_binary(command[0]):
-            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) + \
+            PR.warn("\'{0}\' tool does not exist. ".format(command[0]) +
                     "File system will not be created")
             return 1
         if self.options.force:
@@ -851,40 +851,49 @@ class StorageHandle(object):
                 command.insert(1, '-v')
         return misc.run(command, stdout=True)[0]
 
-    def _do_mount(self, volume, options=None):
+    def _do_mount(self, volume, options=None, directory=None):
+        if directory is None:
+            directory = self._mpoint
         try:
-            volume.mount(self._mpoint, options)
+            volume.mount(directory, options)
         except AttributeError:
-            misc.do_mount(volume['real_dev'], self._mpoint, options)
+            misc.do_mount(volume['real_dev'], directory, options)
 
     def check(self, args):
-        '''
+        """
         Check the file system on the volume. FsInfo is used for that purpose,
         except for btrfs.
-        '''
+        """
         err = 0
+        checked = 0
         for fs in args.device:
             print "Checking {0} file system on \'{1}\':".format(fs.fstype,
                                                                 fs.device),
             if fs.mounted:
+                print
                 try:
-                    print
                     if PR.check(PR.FS_MOUNTED, [fs.device, fs.mounted]):
                         misc.do_umount(fs.device)
-                except Exception:
+                except problem.FsMounted:
+                    PR.warn("Unable to check file system " +
+                            "\'{0}\' on volume \'{1}\'".format(fs.fstype,
+                                                               fs.device))
                     continue
             ret = fs.fsck()
+            checked += 1
             err += ret
             if ret:
                 print "FAIL"
             else:
                 print "OK"
+        if checked == 0:
+            PR.error("Nothing was checked")
         if err > 0:
-            PR.warn("Some file system(s) contains errors. Please run " + \
+            PR.warn("Some file system(s) contains errors. Please run " +
                     "the appropriate fsck utility")
 
     def _filter_device_list(self, args, have_size=None, new_size=None):
-        '''
+        """
         Filter the args.device list. Only items which have to be added to
         pool are left in the args.device list. Function returns touple
         (have_size, devices) where have_size is the size of the devices which
@@ -892,7 +901,7 @@ class StorageHandle(object):
         argument. Devices is the list of devices which can be used for volume
         creation, it means that it does not contain devices which are used
         in other pools and are not removed from it in this function.
-        '''
+        """
 
         if have_size is None:
             have_size = 0.0
@@ -913,7 +922,6 @@ class StorageHandle(object):
                     remove_args.items = [self.dev[dev]]
                     if self.remove(remove_args):
                         args.device.append(dev)
-                        have_size += float(self.dev[dev]['dev_size'])
                         changed = True
                     elif new_size is None:
                         PR.error("Device \'{0}\' can not be used!".format(dev))
@@ -932,27 +940,40 @@ class StorageHandle(object):
                 else:
                     devices.remove(dev)
                     continue
+
             if not self.dev[dev] or 'pool_name' not in self.dev[dev]:
-                args.device.append(dev)
-                if not self.dev[dev]:
-                    have_size += misc.get_file_size(dev)
+                # Check signature of existing file system on the device
+                # and ask user whether to use it or not.
+                signature = misc.get_fs_type(dev)
+                if signature and \
+                   not PR.check(PR.EXISTING_FILESYSTEM, [signature, dev]):
+                    devices.remove(dev)
+                    continue
                 else:
-                    have_size += float(self.dev[dev]['dev_size'])
+                    args.device.append(dev)
 
         if changed:
             self.reinit_dev()
 
+        for dev in devices:
+            if not self.dev[dev]:
+                have_size += misc.get_file_size(dev)
+            else:
+                try:
+                    have_size += float(self.dev[dev]['dev_free'])
+                except ValueError:
+                    have_size += float(self.dev[dev]['dev_size'])
+
         return have_size, devices
 
-
     def resize(self, args):
-        '''
+        """
         Resize the volume to the given size. If more devices are provided as
         arguments, it will be added into the pool prior to the volume resize
         only if the space in the pool is not sufficient. That said, only the
         number of devices are added into the pool to be able to cover the
         resize.
-        '''
+        """
         args.pool = self.pool[args.volume['pool_name']]
         vol_size = float(args.volume['vol_size'])
 
@@ -968,6 +989,18 @@ class StorageHandle(object):
 
         fs = True if 'fs_type' in args.volume else False
 
+        if new_size <= 0:
+            PR.error("New volume size \'{0} KB\' is too small".format(new_size))
+
+        if vol_size == new_size:
+            # Try to grow the file system, since there is nothing to
+            # do with the volume itself.
+            if fs:
+                args.volume['fs_info'].resize()
+            else:
+                PR.check(PR.RESIZE_ALREADY_MATCH, [args.volume.name, new_size])
+            return
+
         # Backend might not support pooling
         if args.pool is None:
             pool_free = None
@@ -977,31 +1010,24 @@ class StorageHandle(object):
             pool_name = args.pool.name
 
         have_size, devices = self._filter_device_list(args,
-                                             pool_free,
-                                             new_size)
+                                                      pool_free,
+                                                      new_size)
 
         if have_size < size_change:
             PR.check(PR.RESIZE_NOT_ENOUGH_SPACE,
                      [pool_name, args.volume.name, new_size])
         elif len(args.device) > 0 and new_size > vol_size:
-            self.add(args)
+            self.add(args, True)
 
         if new_size != vol_size:
             args.volume.resize(new_size, fs)
-        else:
-            # Try to grow the file system, since there is nothing to
-            # do with the volume itself.
-            if fs:
-                args.volume['fs_info'].resize()
-            else:
-                PR.check(PR.RESIZE_ALREADY_MATCH, [args.volume.name, new_size])
 
     def create(self, args):
-        '''
+        """
         Create new volume (or subvolume in case of btrfs) using the devices
         provided as arguments. If the device is not in the selected pool, then
         add() is called on the pool prior to create().
-        '''
+        """
         # Get the size in kilobytes
         if args.size:
             args.size = misc.get_real_size(args.size)
@@ -1016,6 +1042,10 @@ class StorageHandle(object):
         else:
             pool_free = 0.0
 
+        # If devices were provided we should only use those
+        if len(devices) > 0:
+            pool_free = None
+
         have_size, devices = self._filter_device_list(args, pool_free,
                                                       args.size)
 
@@ -1034,13 +1064,29 @@ class StorageHandle(object):
         if not args.pool.exists() and len(devices) == 0:
             PR.check(PR.NO_DEVICES, args.pool.name)
 
+        if have_size == 0:
+            PR.error("Not enough space ({0} KB) to".format(have_size) +
+                     "to create volume")
+
+        # Number of stripes must not exceed number of devices within the pool
+        if args.stripes and len(devices) > 0 and args.stripes > len(devices):
+            PR.error("Number of stripes ({0}) ".format(args.stripes) +
+                     "must not exceed number of devices " +
+                     "({0})".format(len(devices)))
+        elif args.stripes and len(devices) == 0 and args.pool.exists():
+            tmp = int(args.pool['dev_count'])
+            if args.stripes > tmp:
+                PR.error("Number of stripes ({0}) ".format(args.stripes) +
+                         "must not exceed number of devices " +
+                         "({0})".format(tmp))
+
         # If we want btrfs pool and it does not exist yet, we do not
         # want to call add since it would create it. Note that when
         # btrfs pool is created the new btrfs volume is created as well
         # because it is actually the same thing
         if len(args.device) > 0 and \
            not (not args.pool.exists() and args.pool.type == 'btrfs'):
-            self.add(args)
+            self.add(args, True)
 
         if args.raid:
             raid = {'level': args.raid,
@@ -1062,9 +1108,9 @@ class StorageHandle(object):
             self._do_mount(self.vol[lvname])
 
     def list(self, args):
-        '''
+        """
         List devices, pools, volumes
-        '''
+        """
         if not args.type:
             self.dev.ptable()
             self.pool.ptable()
@@ -1081,24 +1127,35 @@ class StorageHandle(object):
         elif args.type in ['snap', 'snapshots']:
             self.snap.ptable()
 
-    def add(self, args):
-        '''
+    def add(self, args, skip_check=False):
+        """
         Add devices into the pool
-        '''
-        for dev in args.device[:]:
-            item = self.dev[dev]
-            if item and 'pool_name' in item:
-                if item['pool_name'] == args.pool.name:
-                    args.device.remove(dev)
-                    continue
-                if PR.check(PR.DEVICE_USED, [item.name, item['pool_name']]):
-                    remove_args = Struct()
-                    remove_args.all = False
-                    remove_args.items = [item]
-                    if not self.remove(remove_args):
-                        args.device.remove(item.name)
+        """
+        if not skip_check:
+            for dev in args.device[:]:
+                item = self.dev[dev]
+                if item and 'pool_name' in item:
+                    if item['pool_name'] == args.pool.name:
+                        args.device.remove(dev)
+                        continue
+                    if PR.check(PR.DEVICE_USED,
+                                [item.name, item['pool_name']]):
+                        remove_args = Struct()
+                        remove_args.all = False
+                        remove_args.items = [item]
+                        if not self.remove(remove_args):
+                            args.device.remove(item.name)
+                    else:
+                        args.device.remove(dev)
                 else:
-                    args.device.remove(dev)
+                    # Check signature of existing file system on the device
+                    # and as user whether to use it or not.
+                    signature = misc.get_fs_type(dev)
+                    if signature and \
+                       not PR.check(PR.EXISTING_FILESYSTEM, [signature, dev]):
+                        args.device.remove(dev)
+                        continue
+
         if args.pool.exists():
             if len(args.device) > 0:
                 args.pool.extend(args.device)
@@ -1111,15 +1168,23 @@ class StorageHandle(object):
                 PR.check(PR.NO_DEVICES, args.pool.name)
 
     def remove(self, args):
-        '''
+        """
         Remove the all the items, or all pools if all argument is specified.
         Items could be the devices, pools or volumes.
-        '''
+        """
         ret = True
+        removed = 0
         if args.all:
             for pool in self.pool:
-                pool.remove()
-            return
+                try:
+                    pool.remove()
+                    removed += 1
+                except (RuntimeError, problem.SsmError), ex:
+                    PR.info("Unable to remove '{0}'".format(pool['pool_name']))
+                    ret = False
+            if removed == 0:
+                PR.error("Nothing was removed!")
+            return ret
         if len(args.items) == 0:
             err = "too few arguments"
             raise argparse.ArgumentTypeError(err)
@@ -1129,24 +1194,28 @@ class StorageHandle(object):
                     pool = self.pool[item['pool_name']]
                     if pool:
                         pool.reduce(item.name)
+                        removed += 1
                         continue
                     else:
                         PR.error("It is not clear what do you want " +
                                  "to achieve by removing " +
                                  "{0}!".format(item.name))
                 item.remove()
+                removed += 1
             except (RuntimeError, problem.SsmError), ex:
                 PR.info("Unable to remove '{0}'".format(item.name))
                 ret = False
+        if removed == 0:
+            PR.error("Nothing was removed")
         return ret
 
     def snapshot(self, args):
-        '''
+        """
         Create a new snapshot of the volume.
-        '''
+        """
         pool = self.pool[args.volume['pool_name']]
         vol_size = float(args.volume['vol_size'])
-        pool_size = float(pool['pool_size'])
+        pool_free = float(pool['pool_free'])
 
         if not args.size:
         # We'll ceate snapshot of the size of 20% of the original volume
@@ -1156,11 +1225,30 @@ class StorageHandle(object):
             snap_size = float(misc.get_real_size(args.size))
             user_set_size = True
 
-        if pool_size < snap_size:
-            snap_size = pool_size
+        if pool_free < snap_size:
+            snap_size = pool_free
+
+        if snap_size <= 0 and pool.type != 'btrfs':
+            PR.error("Not enough space ({0} KB) to".format(pool_free) +
+                     "to create snapshot")
 
         args.volume.snapshot(args.dest, args.name, snap_size, user_set_size)
 
+    def mount(self, args):
+        """
+        Mount a volume at given directory.
+        """
+        vol = self.vol[args.volume]
+        try:
+            if vol:
+                self._do_mount(vol, args.options, args.directory)
+            else:
+                misc.do_mount(args.volume, args.directory, args.options)
+        except RuntimeError:
+            PR.error("Could not mount {0} to ".format(args.volume) +
+                     "{0} with options \'{1}\'".format(args.directory,
+                                                       args.options))
+
     def is_fs(self, device):
         real = misc.get_real_device(device)
 
@@ -1174,12 +1262,12 @@ class StorageHandle(object):
         raise argparse.ArgumentTypeError(err)
 
     def _find_device_record(self, path):
-        '''
+        """
         Try to find device name for path, which is used as an key in
         self.dev - this is usually the real block device, but in some
         rare cases (dmsetup) we can have real block device which name
         does not correspond with what we have in /proc/partitions
-        '''
+        """
         if self.dev[path]:
             return path
 
@@ -1191,9 +1279,9 @@ class StorageHandle(object):
             return path
 
     def check_create_item(self, path):
-        '''
+        """
         Check the create argument for block device or directory.
-        '''
+        """
         if not self._mpoint:
             try:
                 mode = os.stat(path).st_mode
@@ -1203,8 +1291,7 @@ class StorageHandle(object):
             if stat.S_ISDIR(mode):
                 self._mpoint = path
                 return
-        path = is_bdevice(path)
-        return self._find_device_record(path)
+        return self.get_bdevice(path)
 
     def get_bdevice(self, path):
         path = is_bdevice(path)
@@ -1253,9 +1340,9 @@ class StorageHandle(object):
                 raise argparse.ArgumentTypeError(err)
 
     def check_remove_item(self, string):
-        '''
+        """
         Check the remove argument for volume, pool or device.
-        '''
+        """
         volume = self.vol[string]
         if volume:
             return volume
@@ -1267,8 +1354,7 @@ class StorageHandle(object):
             return device
         else:
             try:
-                path = is_bdevice(string)
-                path = self._find_device_record(path)
+                path = self.get_bdevice(string)
                 device = self.dev[path]
                 if device:
                     return device
@@ -1318,6 +1404,19 @@ def valid_resize_size(size):
         raise argparse.ArgumentTypeError(err)
 
 
+def is_directory(self, string):
+    try:
+        mode = os.stat(string).st_mode
+    except OSError:
+        err = "'{0}' does not exist.".format(string)
+        raise argparse.ArgumentTypeError(err)
+    if stat.S_ISDIR(mode):
+        return string
+    else:
+        err = "'{0}' is not directory.".format(string)
+        raise argparse.ArgumentTypeError(err)
+
+
 def is_bdevice(path):
     path = misc.get_real_device(path)
     try:
@@ -1355,6 +1454,7 @@ class SsmParser(object):
         self.parser_add = self._get_parser_add()
         self.parser_remove = self._get_parser_remove()
         self.parser_snapshot = self._get_parser_snapshot()
+        self.parser_mount = self._get_parser_mount()
         self.args = None
 
     def parse(self):
@@ -1375,14 +1475,14 @@ class SsmParser(object):
                 help="Show aditional information while executing.",
                 action="store_true")
         parser.add_argument('-f', '--force',
-                help="Force execution in the case where ssm has some " +\
+                help="Force execution in the case where ssm has some " +
                      "doubts or questions.",
                 action="store_true")
         parser.add_argument('-b', '--backend', nargs=1,
                 metavar='BACKEND',
-                help="Choose backend to use. Currently you can choose from " + \
+                help="Choose backend to use. Currently you can choose from " +
                      "({0}).".format(",".join(SUPPORTED_BACKENDS)),
-                choices = SUPPORTED_BACKENDS,
+                choices=SUPPORTED_BACKENDS,
                 action=SetBackend)
         return parser
 
@@ -1455,11 +1555,13 @@ class SsmParser(object):
                      following list of supported levels
                      ({0}).'''.format(",".join(SUPPORTED_RAID)))
         parser_create.add_argument('-I', '--stripesize',
+                type=int,
                 help='''Gives the number of kilobytes for the granularity
                         of stripes. This is optional and if not given, backend
                         default will be used. Note that you have to specify RAID
                         level as well.''')
         parser_create.add_argument('-i', '--stripes',
+                type=int,
                 help='''Gives the number of stripes. This is equal to the
                      number of physical volumes to scatter the logical
                      volume. This is optional and if stripesize is set
@@ -1505,7 +1607,8 @@ class SsmParser(object):
                      pool is used.''', type=self.storage.is_pool)
         parser_add.add_argument('device', nargs='+',
                 help="Devices to add into the pool.",
-                type=self.storage.get_bdevice)
+                type=self.storage.get_bdevice,
+                action=StoreAll)
         parser_add.set_defaults(func=self.storage.add)
         return parser_add
 
@@ -1551,6 +1654,24 @@ class SsmParser(object):
         parser_snapshot.set_defaults(func=self.storage.snapshot)
         return parser_snapshot
 
+    def _get_parser_mount(self):
+        """
+        Mount command
+        """
+        parser_mount = self.subcommands.add_parser("mount",
+                help='''Mount a volume with file system to specified
+                     locaion.''')
+        parser_mount.add_argument('-o', '--options',
+                help='''Options are specified with a -o flag followed by a
+                     comma separated string of options. This option is
+                     equivalent to the same mount(8) option.''')
+        parser_mount.add_argument("volume", help="Volume to mount.")
+        parser_mount.add_argument("directory",
+                help="Directory to attach the volume.",
+                type=is_directory)
+        parser_mount.set_defaults(func=self.storage.mount)
+        return parser_mount
+
 
 def main(args=None):
 
@@ -1558,6 +1679,7 @@ def main(args=None):
         sys.argv = args.split()
 
     options = Options()
+    PR.set_options(options)
     storage = StorageHandle(options)
     ssm_parser = SsmParser(storage)
     args = ssm_parser.parse()
@@ -1579,7 +1701,6 @@ def main(args=None):
 
     #storage.set_globals(args.force, args.verbose, args.yes, args.config)
     storage.set_globals(options)
-    PR.set_options(options)
 
     # Register clean-up function on exit
     sys.exitfunc = misc.do_cleanup
