@@ -27,13 +27,14 @@ import unittest
 import argparse
 from ssmlib import main
 from ssmlib import misc
+from ssmlib import problem
 
 from tests.unittests.common import *
 
 class SimpleStorageHandleSanityCheck(BaseStorageHandleInit):
-    '''
+    """
     Simple sanity ckecks for StorageHandle class and some of its methods.
-    '''
+    """
 
     def test_constructor(self):
         # Check initial variables
@@ -99,10 +100,10 @@ class SimpleStorageHandleSanityCheck(BaseStorageHandleInit):
 
 
 class StorageHandleSanityCheck(BaseStorageHandleInit):
-    '''
+    """
     Sanity checks for StorageHandle class and other classes used via
     StorageHandle class.
-    '''
+    """
 
     def setUp(self):
         super(StorageHandleSanityCheck, self).setUp()
@@ -245,9 +246,9 @@ class StorageHandleSanityCheck(BaseStorageHandleInit):
 
 
 class SimpleSsmSanityCheck(unittest.TestCase):
-    '''
+    """
     Simple sanity check of ssm module.
-    '''
+    """
 
     def test_existing_objects(self):
         obj = dir(main)
@@ -283,13 +284,14 @@ class SimpleSsmSanityCheck(unittest.TestCase):
         self.assert_("add" in obj)
         self.assert_("remove" in obj)
         self.assert_("snapshot" in obj)
+        self.assert_("mount" in obj)
         self.assert_("set_globals" in obj)
 
 
 class SsmFunctionCheck(MockSystemDataSource):
-    '''
+    """
     Prepare data for ssm function check.
-    '''
+    """
 
     def setUp(self):
         super(SsmFunctionCheck, self).setUp()
@@ -315,8 +317,11 @@ class SsmFunctionCheck(MockSystemDataSource):
 
     def mock_run(self, cmd, *args, **kwargs):
         for i,arg in enumerate(cmd):
-            if arg == None:
+            if arg is None:
                 cmd[i] = ''
+            elif type(arg) is not str:
+                cmd[i] = str(arg)
+
         self.run_data.append(re.sub('\s\s+',' '," ".join(cmd).strip()))
         output = ""
         if cmd[0] == 'pooldata':
@@ -341,37 +346,37 @@ class SsmFunctionCheck(MockSystemDataSource):
 
         # Extend Volume
         self._checkCmd("ssm resize", ['--size +4m', '/dev/default_pool/vol003'],
-            "vol resize /dev/default_pool/vol003 5120.0 False");
+            "vol resize /dev/default_pool/vol003 5120.0 False")
 
         # Shrink volume
         self._checkCmd("ssm resize", ['-s-100G', '/dev/default_pool/vol002'],
-            "vol resize /dev/default_pool/vol002 132426625.0 False");
+            "vol resize /dev/default_pool/vol002 132426625.0 False")
 
         # Set volume size
         self._checkCmd("ssm resize", ['-s 10M', '/dev/my_pool/vol001'],
-            "vol resize /dev/my_pool/vol001 10240.0 False");
+            "vol resize /dev/my_pool/vol001 10240.0 False")
 
         # Set volume and add devices
         self._checkCmd("ssm resize -s 12T /dev/default_pool/vol003 /dev/sdc1 /dev/sde",
-            [], "vol resize /dev/default_pool/vol003 12884901888.0 False");
+            [], "vol resize /dev/default_pool/vol003 12884901888.0 False")
         self.assertEqual(self.run_data[-2],
             "pool extend default_pool /dev/sdc1 /dev/sde")
 
         # Set volume size
         self._checkCmd("ssm resize -s 10G /dev/default_pool/vol003 /dev/sdc1 /dev/sde",
-            [], "vol resize /dev/default_pool/vol003 10485760.0 False");
+            [], "vol resize /dev/default_pool/vol003 10485760.0 False")
         self.assertEqual(self.run_data[-2],
             "pool extend default_pool /dev/sdc1 /dev/sde")
 
         # Extend volume size with adding more devices
         self._checkCmd("ssm resize -s +10G /dev/default_pool/vol003 /dev/sdc1 /dev/sde",
-            [], "vol resize /dev/default_pool/vol003 10486784.0 False");
+            [], "vol resize /dev/default_pool/vol003 10486784.0 False")
         self.assertEqual(self.run_data[-2],
             "pool extend default_pool /dev/sdc1 /dev/sde")
 
         # Shrink volume with devices provided
         self._checkCmd("ssm resize -s-10G /dev/default_pool/vol002 /dev/sdc1 /dev/sde",
-            [], "vol resize /dev/default_pool/vol002 226798465.0 False");
+            [], "vol resize /dev/default_pool/vol002 226798465.0 False")
         self.assertNotEqual(self.run_data[-2],
             "pool extend default_pool /dev/sdc1 /dev/sde")
         self.assertNotEqual(self.run_data[-2],
@@ -384,12 +389,9 @@ class SsmFunctionCheck(MockSystemDataSource):
         # If the device we are able to use can cover the size, then
         # it will be resized successfully
         self._checkCmd("ssm resize -s +1.5T /dev/my_pool/vol001 /dev/sdb /dev/sda /dev/sdc1",
-            [], "vol resize /dev/my_pool/vol001 1613595352.0 False");
+            [], "vol resize /dev/my_pool/vol001 1613595352.0 False")
 
     def test_create(self):
-        #out = MyStdout()
-        #sys.stdout = out
-
         # Create volume using single device from non existent default pool
         self._checkCmd("ssm create", ['/dev/sda'],
             "pool create {0} /dev/sda".format(main.DEFAULT_DEVICE_POOL))
@@ -423,8 +425,11 @@ class SsmFunctionCheck(MockSystemDataSource):
             "pool create {0} 2791728742.40 0 16 /dev/sda".format(main.DEFAULT_DEVICE_POOL))
         self._cmdEq("pool new {0} /dev/sda".format(main.DEFAULT_DEVICE_POOL), -2)
 
-        self._checkCmd("ssm create", ['-r 1', '-s 2.6T', '-I 16', '-i 4', '/dev/sda'],
-            "pool create {0} 2791728742.40 1 4 16 /dev/sda".format(main.DEFAULT_DEVICE_POOL))
+        # Number of stripes must not exceed number of devices
+        self.assertRaises(problem.GeneralError, main.main, "ssm create -r 1 -s 2.6T -I 16 -i 4 /dev/sda")
+
+        self._checkCmd("ssm create", ['-r 1', '-s 2.6T', '-I 16', '/dev/sda'],
+            "pool create {0} 2791728742.40 1 16 /dev/sda".format(main.DEFAULT_DEVICE_POOL))
         self._cmdEq("pool new {0} /dev/sda".format(main.DEFAULT_DEVICE_POOL), -2)
 
         # Create volume using single device from non existent my_pool
@@ -441,7 +446,7 @@ class SsmFunctionCheck(MockSystemDataSource):
         self._cmdEq("pool new my_pool /dev/sda", -2)
 
         self._checkCmd("ssm create", ['-r 0', '-p my_pool', '-s 2.6T', '-I 16',
-            '-i 4', '/dev/sda'], "pool create my_pool 2791728742.40 0 4 16 /dev/sda")
+            '/dev/sda'], "pool create my_pool 2791728742.40 0 16 /dev/sda")
         self._cmdEq("pool new my_pool /dev/sda", -2)
 
         # Create volume using multiple devices
@@ -456,27 +461,27 @@ class SsmFunctionCheck(MockSystemDataSource):
         # Create volume using single device from existing pool
         self._addPool(main.DEFAULT_DEVICE_POOL, ['/dev/sdb', '/dev/sdd'])
         self._checkCmd("ssm create", ['-r 10', '-s 2.6T', '-I 16',
-            '-i 4', '-n myvolume', '/dev/sda'],
-            "pool create {0} 2791728742.40 myvolume 10 4 16 /dev/sda". format(main.DEFAULT_DEVICE_POOL))
+            '-n myvolume', '/dev/sda'],
+            "pool create {0} 2791728742.40 myvolume 10 16 /dev/sda". format(main.DEFAULT_DEVICE_POOL))
         self._cmdEq("pool extend {0} /dev/sda".format(main.DEFAULT_DEVICE_POOL), -2)
 
         self._addPool("my_pool", ['/dev/sdc2', '/dev/sdc3'])
         self._checkCmd("ssm create", ['-r 1', '-p my_pool', '-s 2.6T', '-I 16',
-            '-i 4', '-n myvolume', '/dev/sda'],
-            "pool create my_pool 2791728742.40 myvolume 1 4 16 /dev/sda")
+            '-n myvolume', '/dev/sda'],
+            "pool create my_pool 2791728742.40 myvolume 1 16 /dev/sda")
         self._cmdEq("pool extend my_pool /dev/sda", -2)
 
         # Create volume using multiple devices which one of the is in already
         # in the pool
         self._checkCmd("ssm create", ['-r 0', '-s 2.6T', '-I 16',
-            '-i 4', '-n myvolume', '/dev/sda /dev/sdb'],
-            "pool create {0} 2791728742.40 myvolume 0 4 16 /dev/sda /dev/sdb". format(main.DEFAULT_DEVICE_POOL))
+            '-i 2', '-n myvolume', '/dev/sda /dev/sdb'],
+            "pool create {0} 2791728742.40 myvolume 0 2 16 /dev/sda /dev/sdb". format(main.DEFAULT_DEVICE_POOL))
         self._cmdEq("pool extend {0} /dev/sda".format(main.DEFAULT_DEVICE_POOL), -2)
 
         self._addPool("my_pool", ['/dev/sdc2', '/dev/sdc3'])
         self._checkCmd("ssm create", ['-r 10', '-p my_pool', '-s 2.6T', '-I 16',
-            '-i 4', '-n myvolume', '/dev/sdc2 /dev/sda'],
-            "pool create my_pool 2791728742.40 myvolume 10 4 16 /dev/sdc2 /dev/sda")
+            '-i 2', '-n myvolume', '/dev/sdc2 /dev/sda'],
+            "pool create my_pool 2791728742.40 myvolume 10 2 16 /dev/sdc2 /dev/sda")
         self._cmdEq("pool extend my_pool /dev/sda", -2)
 
         # Test that we do not use devices which are already used in different
@@ -671,6 +676,30 @@ class SsmFunctionCheck(MockSystemDataSource):
             ['--size 1G', '--name test' ,'/mnt/test'],
             "vol snapshot /dev/default_pool/vol004 test 1048576.0 True")
 
+    def test_mount(self):
+        self._addDir("/mnt/test")
+        self._addDir("/mnt/test1")
+        self._addDir("/mnt/test2")
+        # Generate some storage data
+        self._addPool('default_pool', ['/dev/sda', '/dev/sdb'])
+        self._addPool('my_pool', ['/dev/sdc2', '/dev/sdc3', '/dev/sdc1'])
+        self._addVol('vol001', 117283225, 1, 'default_pool', ['/dev/sda'],
+                    '/mnt/test1')
+        self._addVol('vol002', 237284225, 1, 'my_pool', ['/dev/sda'])
+
+        # Simple mount test
+        main.main("ssm mount /dev/default_pool/vol001 /mnt/test")
+        self._cmdEq("mount /dev/default_pool/vol001 /mnt/test")
+        main.main("ssm mount -o discard /dev/default_pool/vol001 /mnt/test")
+        self._cmdEq("mount -o discard /dev/default_pool/vol001 /mnt/test")
+        main.main("ssm mount --options rw,discard,neco=44 /dev/my_pool/vol002 /mnt/test1")
+        self._cmdEq("mount -o rw,discard,neco=44 /dev/my_pool/vol002 /mnt/test1")
+
+        # Non existing volume
+        main.main("ssm mount nonexisting /mnt/test1")
+        self._cmdEq("mount nonexisting /mnt/test1")
+        main.main("ssm mount -o discard,rw nonexisting /mnt/test1")
+        self._cmdEq("mount -o discard,rw nonexisting /mnt/test1")
 
 
 class MyInfo(object):
